/*
 * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package com.oracle.graal.nodes;

import com.oracle.graal.api.meta.ResolvedJavaMethod;
import com.oracle.graal.nodeinfo.GeneratedNode;
import com.oracle.graal.nodes.java.MonitorIdNode;
import com.oracle.graal.nodes.virtual.EscapeObjectState;
import java.util.List;

@GeneratedNode(FrameState.class)
class FrameStateGen extends FrameState {

    public FrameStateGen(FrameState outerFrameState, ResolvedJavaMethod method, int bci, List<ValueNode> values, int localsSize, int stackSize, boolean rethrowException, boolean duringCall,
                List<MonitorIdNode> monitorIds, List<EscapeObjectState> virtualObjectMappings) {
        super(outerFrameState, method, bci, values, localsSize, stackSize, rethrowException, duringCall, monitorIds, virtualObjectMappings);
    }

    public FrameStateGen(int bci) {
        super(bci);
    }

    public FrameStateGen(ResolvedJavaMethod method, int bci, ValueNode[] locals, List<ValueNode> stack, ValueNode[] locks, MonitorIdNode[] monitorIds, boolean rethrowException, boolean duringCall) {
        super(method, bci, locals, stack, locks, monitorIds, rethrowException, duringCall);
    }

    protected int computeTheMeaningOfLife() {
        // this method got partially evaluated
        return 42;
    }

}
